#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "quiz.h"
#include "answer.h"
#include "readField.h"

#define SEPARATOR ';'

//Lettura risposta singolo quiz
Answer* readAnswer(Quiz *quiz);

//Valutazione esame completo
float doAssessment(QuizList quiz);

Quiz* createQuiz()
{
	Quiz *q = (Quiz*)malloc(sizeof(Quiz));
	q->text[0] = '\0';
	q->answerCount = 0;
	q->answers = NULL;
	return q;
}

void destroyQuiz(Quiz *quiz)
{
	if (quiz != NULL)
	{
		free(quiz->answers);
		free(quiz);
	}
}

QuizList createQuizList(Quiz *q)
{
	QuizList list = (QuizList)malloc(sizeof(QuizListElement));
	list->quiz = q;
	list->next = NULL;
	return list;
}

void destroyQuizList(QuizList list)
{
	if (list != NULL)
	{
		destroyQuizList(list->next);
		destroyQuiz(list->quiz);
		free(list);
	}
}


/*
Formato File
NumeroRisposte[int];Domanda[stringa]
Codice[char];Punteggio[float];Risposta[stringa]
Codice[char];Punteggio[float];Risposta[stringa]
Codice[char];Punteggio[float];Risposta[stringa]
*/

Quiz* readQuizFromTxt(FILE *file)
{
	Quiz *quiz = NULL;
	Boolean ok = false;

	QuestionText text;
	int answerCount;
	Answer *answerArray;

	ok = fscanf(file, "%d;", &answerCount) == 1;
	ok = ok && readField(text, '\n', file) == '\n';

	if (ok)
	{
		answerArray = readAnswerArrayFromTxt(file, answerCount);
	}
	if (ok && answerArray != NULL)
	{
		quiz = createQuiz();
		quiz->answerCount = answerCount;
		strcpy(quiz->text, text);
		quiz->answers = answerArray;
	}
	return quiz;
}

QuizList readQuizListFromTxt(FILE *file)
{
	Quiz *quiz = NULL;
	QuizList tail = NULL;
	QuizList head = NULL;

	quiz = readQuizFromTxt(file);
	if (quiz != NULL)
	{
		head = createQuizList(quiz);
	}
	tail = head;
	while (tail != NULL)
	{
		QuizList temp = NULL;
		quiz = readQuizFromTxt(file);
		if (quiz != NULL)
		{
			temp = createQuizList(quiz);
		}
		tail->next = temp;
		tail = temp;
	}
	return head;
}

void renderQuiz(Quiz *quiz)
{
	int i;
	printf("\n%s\n\n", quiz->text);
	for (i = 0; i < quiz->answerCount; i++)
	{
		renderAnswer(quiz->answers + i);
		printf("\n");
	}
	printf("Selezionare la risposta: ");
}

Answer* answerInQuiz(Quiz *quiz, char choice)
{
	int i;
	for (i = 0; i< quiz->answerCount; i++)
	{
		if (choice == quiz->answers[i].code)
		{
			return quiz->answers + i;
		}
	}
	return NULL;
}

Answer* readAnswerFromUser(Quiz *quiz)
{
	Answer *answer = NULL;
	char choice;
	do
	{
		choice = (char)getchar();
		while (getchar() != 10);
		answer = answerInQuiz(quiz, choice);
		if (answer == NULL)
		{
			printf("Scegliere una risposta valida!\n");
		}
	}
	while (answer == NULL);
	return answer;
}

void printAll(QuizList list)
{
	while (list != NULL)
	{
		int i;
		printf("%s\n\n", list->quiz->text);
		for (i = 0; i < list->quiz->answerCount; i++)
		{
			renderAnswer(list->quiz->answers + i);
			printf("\n");
		}
		list = list->next;
	}
}

float doAssessment(QuizList list)
{
	if (list != NULL)
	{
		Answer *answer;
		renderQuiz(list->quiz);
		answer = readAnswerFromUser(list->quiz);
		return answer->points + doAssessment(list->next);
	}
	else
	{
		return 0.0;
	}
}
